@@ -1,21 +1,12 @@
-# encoding: utf-8
-"""
-@author: zyl
-@file: sundat_match.py
-@time: 2021/11/29 9:33
-@desc:
-"""
-
-
-def sunday_match(target, pattern):
+def sunday_match(target, pattern) -> list:
     """
-
+    find all pattern's starts,在一个序列中找到要的所有的子序列
     Args:
-        target:
-        pattern:
+        target: str or list ，原始序列
+        pattern: str or list，要匹配的序列
 
     Returns:
-
+        starts: 匹配到子序列在原始序列中的起始位置
     """
     len_target = len(target)
     len_pattern = len(pattern)
@@ -42,9 +33,9 @@ def sunday_match(target, pattern):
 
 if __name__ == '__main__':
     t = "this is an apple , apple app app is app not app"
-    t = t.split()
-    p = "app not"
-    p = p.split()
+    # t = t.split()
+    p = "app"
+    # p = p.split()
     print(t)
     print(p)
     print(sunday_match(target=t, pattern=p))
