@@ -1,10 +1,3 @@
-# encoding: utf-8
-"""
-@author: zyl
-@file: ner_model.py
-@time: 2021/11/25 13:59
-@desc:
-"""
 import time
 
 import pandas as pd
@@ -12,6 +5,8 @@ import wandb
 from loguru import logger
 from simpletransformers.ner import NERModel
 
+from ..metrics.ner_metric import entity_recognition_v2
+
 
 class NerBIO:
     """
@@ -85,7 +80,7 @@ class NerBIO:
         df = df.astype({'sentence_id': 'int', 'words': 'str', 'labels': 'str'})
         return df
 
-    def train(self, train_data: pd.DataFrame, eval_data: pd.DataFrame,wandb_log=None):
+    def train(self, train_data: pd.DataFrame, eval_data: pd.DataFrame, wandb_log=None):
         # deal with dt
         train_data = NerBIO.deal_with_df(train_data)
         eval_data = NerBIO.deal_with_df(eval_data)
@@ -117,7 +112,7 @@ class NerBIO:
             logger.info(f'start training: model_version---{self.model_version},train_size---{train_size}')
             model.train_model(train_data=train_data, eval_data=eval_data)
             logger.info('training finished!!!')
-            wandb.log( {'train_size': train_size, 'eval_size': eval_size})
+            wandb.log({'train_size': train_size, 'eval_size': eval_size})
             if wandb_log:
                 wandb.log(wandb_log)
             end_time = time.time()
@@ -161,43 +156,38 @@ class NerBIO:
             entities.append(e)
         return entities
 
-    def eval(self, eval_df: pd.DataFrame, ner_t5_metric=False,wandb_log=None):
+    def eval(self, eval_df: pd.DataFrame, ner_t5_metric=False, wandb_log=None):
         eval_data = NerBIO.deal_with_df(eval_df)
         eval_size = len(set(eval_df['sentence_id'].tolist()))
 
-        # self.model_args.update(
-        #     {
-        #         'wandb_project': self.wandb_proj,
-        #         'wandb_kwargs': {
-        #             'name': self.model_version + time.strftime("_%m%d_%H:%M:%S", time.localtime()),
-        #             'tags': [self.model_version, 'eval']
-        #         }
-        #     }
-        # )
-
         # wand_b
         wandb.init(project=self.wandb_proj, config=self.model_args,
                    name=self.model_version + time.strftime("_%m%d_%H:%M:%S", time.localtime()),
                    tags=[self.model_version, 'eval'])
 
         model = NERModel(model_type=self.model_type, model_name=self.model_args.get('best_model_dir'),
-                         args=self.model_args, use_cuda=self.use_cuda, cuda_device=self.cuda_device)
-
+                         args=self.model_args, use_cuda=self.use_cuda, cuda_device=self.cuda_device,
+                         labels=self.labels)
         result, model_outputs, preds_list = model.eval_model(eval_data)
+        if wandb_log:
+            wandb.log(wandb_log)
+        wandb.log({"f1_score": result.get('f1_score'), 'eval_size': eval_size})
 
         if ner_t5_metric:
+            all_entities_cls = set()
+            for c in self.labels:
+                if c.startswith('B'):
+                    all_entities_cls.add(c.split('-')[-1])
+
             labels = eval_data.groupby(by=['sentence_id'], sort=False)
             labels = labels.apply(lambda x: x['labels'].tolist())
 
-            preds_list = [set(NerBIO._get_id_entity(p)) for p in preds_list]
-            labels = [set(NerBIO._get_id_entity(l)) for l in labels]
-            from zyl_utils.model_utils.metrics.ner_metric import entity_recognition_v2
-            res_df = entity_recognition_v2(labels, preds_list)
-            wandb.log({"ner_t5_metric": res_df.iloc[2,-1]})
-
-        wandb.log({"f1_score": result.get('f1_score'),'eval_size':eval_size})
-        if wandb_log:
-            wandb.log(wandb_log)
+            for c in all_entities_cls:
+                y_pred = [set(NerBIO._get_id_entity(p, label=c)) for p in preds_list]
+                y_true = [set(NerBIO._get_id_entity(l, label=c)) for l in labels]
+                print(c+": \n")
+                res_df = entity_recognition_v2(y_true, y_pred)
+                wandb.log({c + "_" + "ner_t5_metric": res_df.iloc[2, -1]})
 
     @staticmethod
     def predict(model, to_predict):
@@ -205,30 +195,30 @@ class NerBIO:
 
 
 if __name__ == '__main__':
-    class Trainer(NerBIO):
+    from zyl_utils import get_best_cuda_device
+
+
+    class M(NerBIO):
         def __init__(self):
-            super(Trainer, self).__init__()
-            self.wandb_proj = 'panel_entity_recognition'
+            super(M, self).__init__()
+            self.wandb_proj = 'test'
+            self.use_cuda = True
+            self.cuda_device = get_best_cuda_device()
+            self.save_dir = './'
 
         def train_sample(self):
             train_file = './test.xlsx'
             eval_file = './test.xlsx'
             train_df = pd.read_excel(train_file)  # type:pd.DataFrame
             eval_df = pd.read_excel(eval_file)  # type:pd.DataFrame
-            print(len(train_df))
-            print(len(eval_df))
-            self.save_dir = './'
+
             self.model_version = 'v0.0.0.0'
             self.model_type = 'bert'
             self.pretrained_model = 'bert-base-multilingual-cased'  # 预训练模型位置 model_name
-            self.use_cuda = True
-            self.cuda_device = 0
 
             self.model_args = self.my_config()
             self.model_args.update(
                 {
-                    'train_file': train_file,
-                    'eval_file': eval_file,
                     'num_train_epochs': 3,
                     'learning_rate': 3e-4,
                     'train_batch_size': 24,  # 28
@@ -238,34 +228,20 @@ if __name__ == '__main__':
                 }
             )
             self.labels = ["O", "B-DISEASE", "I-DISEASE"]
-            self.train(train_df, eval_df)
-
-
-    class Evaluator(NerBIO):
-        def __init__(self):
-            super(Evaluator, self).__init__()
-            self.wandb_proj = 'panel_entity_recognition'
+            self.train(train_df, eval_df, wandb_log=None)
 
         def eval_sample(self):
             eval_file = './test.xlsx'
             eval_data = pd.read_excel(eval_file)
 
-            self.save_dir = './'
             self.model_version = 'erv4.2.0.2'
             self.model_type = 'bert'
-            self.use_cuda = True
-            self.cuda_device = 1
 
             self.model_args = self.my_config()
             self.model_args.update(
                 {
-                    'eval_file': eval_file,
+                    # 'best_model_dir':'./',
                     'eval_batch_size': 16,
-                    'max_seq_length': 512,
                 }
             )
-            self.eval(eval_data)
-
-
-    print(NerBIO._get_id_entity(['B-DISEASE', 'I-DISEASE','O','O','B-DISEASE',
-                                 'B-DISEASE', 'I-DISEASE', 'B-DISEASE',]))
\ No newline at end of file
+            self.eval(eval_data, ner_t5_metric=True, wandb_log={'eval_file': eval_file})
