@@ -5,7 +5,7 @@ import wandb
 from loguru import logger
 from simpletransformers.t5 import T5Model
 
-from zyl_utils.model_utils.metrics import ner_metric
+from ..metrics.ner_metric import entity_recognition_v2
 
 
 class NerT5:
@@ -88,7 +88,7 @@ class NerT5:
             'top_p': None,
         }
 
-    def _deal_with_df(self, data, sliding_window=False, delimiter='|'):
+    def _deal_with_df(self, data, sliding_window=False, delimiter='|', up_sampling=False):
         data = data[['prefix', 'input_text', 'target_text']]
         data = data.astype('str')
 
@@ -116,15 +116,24 @@ class NerT5:
                     res.append(t)
 
             data['target_text'] = res
+
+        if up_sampling:
+            pos_data = data[data['target_text'] != '|']
+            from sklearn.utils import resample
+            up_sampling_data = resample(pos_data, replace=True, n_samples=(len(data) - len(pos_data) - len(pos_data)))
+            data = pd.concat([data, up_sampling_data], ignore_index=True)
+            data = resample(data, replace=False)
         data.dropna(inplace=True)
         return data
 
-    def train(self, train_data: pd.DataFrame, eval_data: pd.DataFrame, sliding_window=False, wandb_log=None):
+    def train(self, train_data: pd.DataFrame, eval_data: pd.DataFrame, sliding_window=False, up_sampling=False,
+              wandb_log=None):
         # deal with dt
         train_raw_size = train_data.shape[0]
         eval_raw_size = eval_data.shape[0]
         logger.info('processing data...')
-        train_data = self._deal_with_df(train_data, sliding_window=sliding_window, delimiter='|')
+        train_data = self._deal_with_df(train_data, sliding_window=sliding_window, delimiter='|',
+                                        up_sampling=up_sampling)
         eval_data = self._deal_with_df(eval_data, sliding_window=sliding_window, delimiter='|')
 
         train_size = train_data.shape[0]
@@ -164,7 +173,7 @@ class NerT5:
 
     def eval(self, eval_data: pd.DataFrame, check_in_input_text: bool = False, delimiter='|',
              tokenizer=None, use_sliding_window=False, sliding_window=None, stride=0.8,
-             pos_neg_ratio=None, use_multi_gpus=None, self_metric=False,wandb_log=None):
+             pos_neg_ratio=None, use_multi_gpus=None, self_metric=False, wandb_log=None):
         # deal_with_dt
         eval_data = self._deal_with_df(eval_data, sliding_window=False)
         eval_size = eval_data.shape[0]
@@ -255,13 +264,13 @@ class NerT5:
             y_true = prefix_df['true_target_text'].tolist()
             y_pred = prefix_df['pred_target_text'].tolist()
             print(f'{prefix} report:')
-            res_df = ner_metric.entity_recognition_v2(y_true, y_pred, pos_neg_ratio=pos_neg_ratio,
-                                                      self_metric=self_metric)
+            res_df = entity_recognition_v2(y_true, y_pred, pos_neg_ratio=pos_neg_ratio,
+                                           self_metric=self_metric)
             eval_res[prefix] = res_df
 
         print(f'sum report:')
-        res_df = ner_metric.entity_recognition_v2(revised_target_texts, revised_pred_target_texts,
-                                                  pos_neg_ratio=pos_neg_ratio, self_metric=self_metric)
+        res_df = entity_recognition_v2(revised_target_texts, revised_pred_target_texts,
+                                       pos_neg_ratio=pos_neg_ratio, self_metric=self_metric)
         eval_res['ner_t5_metric'] = res_df
 
         return eval_res  # {prefix:res_df},type:dict
@@ -497,30 +506,30 @@ class NerT5:
 
 
 if __name__ == '__main__':
-    class Trainer(NerT5):
+    from zyl_utils import get_best_cuda_device
+
+
+    class M(NerT5):
         def __init__(self):
-            super(Trainer, self).__init__()
-            self.wandb_proj = 'panel_entity_recognition'
+            super(M, self).__init__()
+            self.wandb_proj = 'test'
+            self.save_dir = './'
+            self.model_type = 'mt5'  # t5
+            self.use_cuda = True
+            self.cuda_device = get_best_cuda_device()
 
         def train_sample(self):
             train_file = './test.xlsx'
             eval_file = './test.xlsx'
             train_df = pd.read_excel(train_file)  # type:pd.DataFrame
             eval_df = pd.read_excel(eval_file)  # type:pd.DataFrame
-            print(len(train_df))
-            print(len(eval_df))
-            self.save_dir = './'
+
             self.model_version = 'v0.0.0.0'
-            self.model_type = 'mt5'
             self.pretrained_model = 'google/mt5-base'  # 预训练模型位置 model_name
-            self.use_cuda = True
-            self.cuda_device = 0
 
             self.model_args = self.my_config()
             self.model_args.update(
                 {
-                    'train_file': train_file,
-                    'eval_file': eval_file,
                     'num_train_epochs': 3,
                     'learning_rate': 3e-4,
                     'train_batch_size': 24,  # 28
@@ -529,30 +538,19 @@ if __name__ == '__main__':
                     'max_seq_length': 512,
                 }
             )
-            self.train(train_df, eval_df)
-
-
-    class Evaluator(NerT5):
-        def __init__(self):
-            super(Evaluator, self).__init__()
-            self.wandb_proj = 'panel_entity_recognition'
+            self.train(train_df, eval_df, sliding_window=True,
+                       wandb_log={'train_file': train_file, 'eval_file': eval_file})
 
         def eval_sample(self):
             eval_file = './test.xlsx'
             eval_data = pd.read_excel(eval_file)
 
-            self.save_dir = './'
             self.model_version = 'erv0.0.0.0'
-            self.model_type = 'mt5'
-            self.use_cuda = True
-            self.cuda_device = 1
-
             self.model_args = self.my_config()
             self.model_args.update(
                 {
-                    'eval_file': eval_file,
                     'eval_batch_size': 16,
-                    'max_seq_length': 512,
+                    # 'best_model_dir':'./'
                 }
             )
             self.eval(eval_data, check_in_input_text=False, delimiter='|',
